import { HttpClient } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { decode } from 'vlq';
import { Injectable, defineInjectable, inject } from '@angular/core';
import { of } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgPosition {
    /**
     * @param {?=} fileName
     * @param {?=} lineNumber
     * @param {?=} columnNumber
     */
    constructor(fileName = 'unknown', lineNumber = 0, columnNumber = 0) {
        this.fileName = fileName;
        this.lineNumber = lineNumber;
        this.columnNumber = columnNumber;
    }
    /**
     * @param {?} str
     * @return {?}
     */
    static fromString(str) {
        /** @type {?} */
        const position = new NgPosition();
        /** @type {?} */
        const i = str.lastIndexOf(':');
        if (i >= 0 && i < str.length - 1) {
            position.columnNumber = +str.substring(i + 1);
            /** @type {?} */
            const j = str.lastIndexOf(':', i - 1);
            if (j >= 0 && j < i) {
                position.lineNumber = +str.substring(j + 1, i);
                position.fileName = str.substring(0, j);
            }
        }
        return position;
    }
    /**
     * @return {?}
     */
    toString() {
        return this.fileName + ':' + this.lineNumber + ':' + this.columnNumber;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgSourceMapper {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
        // used to cache source maps
        this.cache = {};
    }
    /*
          Public Functions
         */
    /**
     * @param {?} map
     * @param {?} position
     * @return {?}
     */
    getSourceInfoFromMap(map$$1, position) {
        // => ';' indicates end of a line
        // => ',' separates mappings in a line
        // decoded mapping => [ generatedCodeColumn, sourceFileIndex, sourceCodeLine, sourceCodeColumn, nameIndex ]
        /** @type {?} */
        let sourceFileIndex = 0;
        /** @type {?} */
        let // second field
        sourceCodeLine = 0;
        /** @type {?} */
        let // third field
        sourceCodeColumn = 0;
        // fourth field
        /** @type {?} */
        const lines = map$$1.mappings.split(';');
        for (let lineIndex = 0; lineIndex < lines.length; lineIndex++) {
            // reset column position to 0 after each line
            /** @type {?} */
            let generatedCodeColumn = 0;
            // decode sections in line
            /** @type {?} */
            const columns = lines[lineIndex].split(',');
            for (let columnIndex = 0; columnIndex < columns.length; columnIndex++) {
                /** @type {?} */
                const decodedSection = decode(columns[columnIndex]);
                if (decodedSection.length >= 4) {
                    // update relative positions
                    generatedCodeColumn += decodedSection[0];
                    sourceFileIndex += decodedSection[1];
                    sourceCodeLine += decodedSection[2];
                    sourceCodeColumn += decodedSection[3];
                }
                // check if matching map
                if (lineIndex === position.lineNumber) {
                    if (generatedCodeColumn === position.columnNumber) {
                        // matching column and line found
                        return new NgPosition(map$$1.sources[sourceFileIndex], sourceCodeLine, sourceCodeColumn);
                    }
                    else if (columnIndex + 1 === columns.length) {
                        // matching column not found, but line is correct
                        return new NgPosition(map$$1.sources[sourceFileIndex], sourceCodeLine, 0);
                    }
                }
            }
        }
        // failed if reached
        return new NgPosition('unknown', 0, 0);
    }
    /**
     * @param {?} position
     * @param {?=} cache
     * @return {?}
     */
    getSourceInfoFromURL(position, cache = true) {
        // url check
        if (!position.fileName.startsWith('http')) {
            return of(position);
        }
        // check if we have map has already, otherwise request from server
        if (this.cache.hasOwnProperty(position.fileName)) {
            return of(this.getSourceInfoFromMap(this.cache[position.fileName], position));
        }
        return this.http.get(position.fileName + '.map').pipe(map((/**
         * @param {?} sourceMap
         * @return {?}
         */
        sourceMap => {
            // store file in cache if not already stored
            if (!this.cache.hasOwnProperty(position.fileName) && cache) {
                this.cache[position.fileName] = sourceMap;
            }
            // map generated position to source position
            return this.getSourceInfoFromMap(sourceMap, position);
        })));
    }
}
NgSourceMapper.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
NgSourceMapper.ctorParameters = () => [
    { type: HttpClient }
];
/** @nocollapse */ NgSourceMapper.ngInjectableDef = defineInjectable({ factory: function NgSourceMapper_Factory() { return new NgSourceMapper(inject(HttpClient)); }, token: NgSourceMapper, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgSourceMapperMock {
    constructor() { }
    /*
          Mock Public Functions
         */
    /**
     * @param {?} map
     * @param {?} position
     * @return {?}
     */
    getSourceInfoFromMap(map$$1, position) {
        return new NgPosition('mock.js', 0, 0);
    }
    /**
     * @param {?} position
     * @param {?=} cache
     * @return {?}
     */
    getSourceInfoFromURL(position, cache = true) {
        return of(new NgPosition('mock.js', 0, 0));
    }
}
NgSourceMapperMock.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
NgSourceMapperMock.ctorParameters = () => [];
/** @nocollapse */ NgSourceMapperMock.ngInjectableDef = defineInjectable({ factory: function NgSourceMapperMock_Factory() { return new NgSourceMapperMock(); }, token: NgSourceMapperMock, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgPosition, NgSourceMapper, NgSourceMapperMock };

//# sourceMappingURL=ng-source-mapper.js.map