import { HttpClient } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { decode } from 'vlq';
import { Injectable, defineInjectable, inject } from '@angular/core';
import { of } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgPosition = /** @class */ (function () {
    function NgPosition(fileName, lineNumber, columnNumber) {
        if (fileName === void 0) { fileName = 'unknown'; }
        if (lineNumber === void 0) { lineNumber = 0; }
        if (columnNumber === void 0) { columnNumber = 0; }
        this.fileName = fileName;
        this.lineNumber = lineNumber;
        this.columnNumber = columnNumber;
    }
    /**
     * @param {?} str
     * @return {?}
     */
    NgPosition.fromString = /**
     * @param {?} str
     * @return {?}
     */
    function (str) {
        /** @type {?} */
        var position = new NgPosition();
        /** @type {?} */
        var i = str.lastIndexOf(':');
        if (i >= 0 && i < str.length - 1) {
            position.columnNumber = +str.substring(i + 1);
            /** @type {?} */
            var j = str.lastIndexOf(':', i - 1);
            if (j >= 0 && j < i) {
                position.lineNumber = +str.substring(j + 1, i);
                position.fileName = str.substring(0, j);
            }
        }
        return position;
    };
    /**
     * @return {?}
     */
    NgPosition.prototype.toString = /**
     * @return {?}
     */
    function () {
        return this.fileName + ':' + this.lineNumber + ':' + this.columnNumber;
    };
    return NgPosition;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgSourceMapper = /** @class */ (function () {
    function NgSourceMapper(http) {
        this.http = http;
        // used to cache source maps
        this.cache = {};
    }
    /*
        Public Functions
       */
    /*
          Public Functions
         */
    /**
     * @param {?} map
     * @param {?} position
     * @return {?}
     */
    NgSourceMapper.prototype.getSourceInfoFromMap = /*
          Public Functions
         */
    /**
     * @param {?} map
     * @param {?} position
     * @return {?}
     */
    function (map$$1, position) {
        // => ';' indicates end of a line
        // => ',' separates mappings in a line
        // decoded mapping => [ generatedCodeColumn, sourceFileIndex, sourceCodeLine, sourceCodeColumn, nameIndex ]
        /** @type {?} */
        var sourceFileIndex = 0;
        /** @type {?} */
        var // second field
        sourceCodeLine = 0;
        /** @type {?} */
        var // third field
        sourceCodeColumn = 0;
        // fourth field
        /** @type {?} */
        var lines = map$$1.mappings.split(';');
        for (var lineIndex = 0; lineIndex < lines.length; lineIndex++) {
            // reset column position to 0 after each line
            /** @type {?} */
            var generatedCodeColumn = 0;
            // decode sections in line
            /** @type {?} */
            var columns = lines[lineIndex].split(',');
            for (var columnIndex = 0; columnIndex < columns.length; columnIndex++) {
                /** @type {?} */
                var decodedSection = decode(columns[columnIndex]);
                if (decodedSection.length >= 4) {
                    // update relative positions
                    generatedCodeColumn += decodedSection[0];
                    sourceFileIndex += decodedSection[1];
                    sourceCodeLine += decodedSection[2];
                    sourceCodeColumn += decodedSection[3];
                }
                // check if matching map
                if (lineIndex === position.lineNumber) {
                    if (generatedCodeColumn === position.columnNumber) {
                        // matching column and line found
                        return new NgPosition(map$$1.sources[sourceFileIndex], sourceCodeLine, sourceCodeColumn);
                    }
                    else if (columnIndex + 1 === columns.length) {
                        // matching column not found, but line is correct
                        return new NgPosition(map$$1.sources[sourceFileIndex], sourceCodeLine, 0);
                    }
                }
            }
        }
        // failed if reached
        return new NgPosition('unknown', 0, 0);
    };
    /**
     * @param {?} position
     * @param {?=} cache
     * @return {?}
     */
    NgSourceMapper.prototype.getSourceInfoFromURL = /**
     * @param {?} position
     * @param {?=} cache
     * @return {?}
     */
    function (position, cache) {
        var _this = this;
        if (cache === void 0) { cache = true; }
        // url check
        if (!position.fileName.startsWith('http')) {
            return of(position);
        }
        // check if we have map has already, otherwise request from server
        if (this.cache.hasOwnProperty(position.fileName)) {
            return of(this.getSourceInfoFromMap(this.cache[position.fileName], position));
        }
        return this.http.get(position.fileName + '.map').pipe(map((/**
         * @param {?} sourceMap
         * @return {?}
         */
        function (sourceMap) {
            // store file in cache if not already stored
            if (!_this.cache.hasOwnProperty(position.fileName) && cache) {
                _this.cache[position.fileName] = sourceMap;
            }
            // map generated position to source position
            return _this.getSourceInfoFromMap(sourceMap, position);
        })));
    };
    NgSourceMapper.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    NgSourceMapper.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    /** @nocollapse */ NgSourceMapper.ngInjectableDef = defineInjectable({ factory: function NgSourceMapper_Factory() { return new NgSourceMapper(inject(HttpClient)); }, token: NgSourceMapper, providedIn: "root" });
    return NgSourceMapper;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgSourceMapperMock = /** @class */ (function () {
    function NgSourceMapperMock() {
    }
    /*
        Mock Public Functions
       */
    /*
          Mock Public Functions
         */
    /**
     * @param {?} map
     * @param {?} position
     * @return {?}
     */
    NgSourceMapperMock.prototype.getSourceInfoFromMap = /*
          Mock Public Functions
         */
    /**
     * @param {?} map
     * @param {?} position
     * @return {?}
     */
    function (map$$1, position) {
        return new NgPosition('mock.js', 0, 0);
    };
    /**
     * @param {?} position
     * @param {?=} cache
     * @return {?}
     */
    NgSourceMapperMock.prototype.getSourceInfoFromURL = /**
     * @param {?} position
     * @param {?=} cache
     * @return {?}
     */
    function (position, cache) {
        if (cache === void 0) { cache = true; }
        return of(new NgPosition('mock.js', 0, 0));
    };
    NgSourceMapperMock.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    NgSourceMapperMock.ctorParameters = function () { return []; };
    /** @nocollapse */ NgSourceMapperMock.ngInjectableDef = defineInjectable({ factory: function NgSourceMapperMock_Factory() { return new NgSourceMapperMock(); }, token: NgSourceMapperMock, providedIn: "root" });
    return NgSourceMapperMock;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgPosition, NgSourceMapper, NgSourceMapperMock };

//# sourceMappingURL=ng-source-mapper.js.map